//           ///          //                                  C++ Cross Platform
//          /////        ////
//         /// XXX     XXX ///            ///////////   /////////     ///   ///
//        ///    XXX XXX    ///         ///             ///    ///   ///  ///
//       ///       XXX       ///         /////////     ///      //  //////
//      ///      XXX XXX      ///               ///   ///    ///   ///  ///
//     ////    XXX     XXX    ////    ////////////  //////////    ///   ///
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//
//
///////
     //
     //  MOBOTIX EventStream Client SDK 
     //
     //  See LICENSE_SDK included in the SDK package
     //
     //  Copyright (c) 2005 - 2016, MOBOTIX AG
     //  All rights reserved.
     //
///////
    

#ifndef M1IMAGE_JSIMAGE_SRC_MXPEG_LOGGER_HPP_
#define M1IMAGE_JSIMAGE_SRC_MXPEG_LOGGER_HPP_

#include <iostream>
#include <sstream>
#include <stdlib.h>

#ifdef __EMSCRIPTEN__
#include <emscripten.h>
#endif

#ifndef MX_LOG_ACTIVE
#define MX_LOG_ACTIVE (MX_LOG_SYSTEM | MX_LOG_NET | MX_LOG_C_API | MX_LOG_DECODER | MX_LOG_EVENT | MX_LOG_VIDEO | MX_LOG_AUDIO)
#endif

#define MX_LOG_SYSTEM           (1<<0)
#define MX_LOG_NET              (1<<1)
#define MX_LOG_VIDEO            (1<<2)
#define MX_LOG_AUDIO            (1<<3)
#define MX_LOG_C_API            (1<<4)
#define MX_LOG_DECODER          (1<<5)
#define MX_LOG_EVENT            (1<<6)
#define MX_LOG_AV_SYNC          (1<<7)
#define MX_LOG_CONFIG           (1<<8)
#define MX_LOG_INPUT            (1<<9)


#define MX_LOG_TARGET_MAIN              (1<<0)
#define MX_LOG_TARGET_AUDIO_WORKER      (1<<1)
#define MX_LOG_TARGET_VIDEO_WORKER      (1<<2)

extern int mx_active_log;

void loggerSetMask(int mask);

inline void logger(int logModule, const char *func, int line, std::ostringstream * oss) {
   if(logModule & mx_active_log) {
      std::cerr << func << ":" << line << ": " << oss->str() << std::endl;
   }
}

#define TIMERFMT(X) "timestamp=" << ((X).tv_sec) << ":" << ((X).tv_usec)

#define MX_LOG_PRINT(X) do { std::cout << X << std::endl; } while (0)
#define MX_LOG_PRINT_VERBOSE(X) do { std::ostringstream oss; oss << X; logger(MX_LOG_MODULE, __PRETTY_FUNCTION__, __LINE__, &oss); } while (0)

#ifdef DEBUG
#define MX_DEBUG(X) MX_LOG_PRINT_VERBOSE(X)
#define MX_ERR(X) MX_LOG_PRINT_VERBOSE(X)
#define MX_INFO(X) MX_LOG_PRINT_VERBOSE(X)
#define MX_WARN(X) MX_LOG_PRINT_VERBOSE(X)
#else
#define MX_DEBUG(X)
#define MX_ERR(X)  MX_LOG_PRINT(X)
#define MX_INFO(X) MX_LOG_PRINT_VERBOSE(X)
#define MX_WARN(X) MX_LOG_PRINT(X)
#endif

#if defined __EMSCRIPTEN__ && defined DEBUG

#define MX_BUFFER_STAT(NAME,VALUE) { \
      EM_ASM_ARGS({ \
         var sName = Pointer_stringify($0); \
         if( typeof(__JSIMAGE_DEBUG) === 'undefined' ) { \
            __JSIMAGE_DEBUG = new Object(); \
         } \
         __JSIMAGE_DEBUG[sName] = $1; \
      }, (NAME), (VALUE) ); \
}

#else
#define MX_BUFFER_STAT(NAME,VALUE) MX_DEBUG("stat: " << (NAME) << " = " << (VALUE))
#endif

#ifdef __EMSCRIPTEN__
#define MX_ALERT(X)  { std::ostringstream oss; oss << X << std::endl;\
   EM_ASM_ARGS({ \
      var msg = Pointer_stringify($0); \
      alert(msg); \
   }, oss.str().c_str()); \
}

#else
#define MX_ALERT(X) MX_INFO(X)
#endif

#endif /* M1IMAGE_JSIMAGE_SRC_MXPEG_LOGGER_HPP_ */
